<?php
/**
 * Created by @exife
 * Website: exife.com
 * Email: support@exife.com
 * Name: Special Promotions (Cart Rules)
 */

class ModelSaleSpecialHints extends Model {

    public function getHints($data) {
        $sql = "SELECT *, DATE(date_start) date_start, DATE(date_end) date_end FROM " . DB_PREFIX . "sp_hint WHERE 1 = 1";

        if (!empty($data['filter_name'])) {
            $sql .= " AND name LIKE '" . $this->db->escape($data['filter_name']) . "%'";
        }

        if (!empty($data['filter_date_start'])) {
            $sql .= " AND DATE(date_start) = DATE('" . $this->db->escape($data['filter_date_start']) . "')";
        }

        if (!empty($data['filter_date_end'])) {
            $sql .= " AND DATE(date_end) = DATE('" . $this->db->escape($data['filter_date_end']) . "')";
        }

        if (isset($data['filter_status'])) {
            $sql .= " AND status = '" . (int)$data['filter_status'] . "'";
        }

        if (isset($data['filter_sort_order'])) {
            $sql .= " AND sort_order = '" . (int)$data['filter_sort_order'] . "'";
        }

        $sql .= " GROUP BY hint_id";

        $sort_data = array(
            'name',
            'date_start',
            'date_end',
            'status',
            'sort_order'
        );

        if (isset($data['sort']) && in_array($data['sort'], $sort_data)) {
            $sql .= " ORDER BY " . $data['sort'];
        } else {
            $sql .= " ORDER BY name";
        }

        if (isset($data['order']) && ($data['order'] == 'DESC')) {
            $sql .= " DESC";
        } else {
            $sql .= " ASC";
        }

        if (isset($data['start']) || isset($data['limit'])) {
            if ($data['start'] < 0) {
                $data['start'] = 0;
            }

            if ($data['limit'] < 1) {
                $data['limit'] = 20;
            }

            $sql .= " LIMIT " . (int)$data['start'] . "," . (int)$data['limit'];
        }

        $query = $this->db->query($sql);

        return $query->rows;
    }

    public function getTotalHints($data) {
        $sql = "SELECT COUNT(DISTINCT hint_id) AS total FROM " . DB_PREFIX . "sp_hint WHERE 1 = 1";

        if (!empty($data['filter_name'])) {
            $sql .= " AND name LIKE '" . $this->db->escape($data['filter_name']) . "%'";
        }

        if (!empty($data['filter_date_start'])) {
            $sql .= " AND DATE(date_start) = DATE('" . $this->db->escape($data['filter_date_start']) . "')";
        }

        if (!empty($data['filter_date_end'])) {
            $sql .= " AND DATE(date_end) = DATE('" . $this->db->escape($data['filter_date_end']) . "')";
        }

        if (isset($data['filter_status'])) {
            $sql .= " AND status = '" . (int)$data['filter_status'] . "'";
        }

        if (isset($data['filter_sort_order'])) {
            $sql .= " AND sort_order = '" . (int)$data['filter_sort_order'] . "'";
        }

        $query = $this->db->query($sql);

        return $query->row['total'];
    }

    public function addHint($data) {

        $this->db->query("
            INSERT INTO
                " . DB_PREFIX . "sp_hint
            SET
                `name` = '" . $this->db->escape($data['name']) . "',
                `status` = '" . (int)$data['status'] . "',
                `date_start` = '" . $this->db->escape($data['date_start']) . "',
                `date_end` = '" . $this->db->escape($data['date_end']) . "',
                `sort_order` = '" . (int)$data['sort_order'] . "',
                `rule` = '" . $this->db->escape(serialize($data['rule'])) . "',
                `logged` = '" . (int)$data['logged'] . "',
                `show_as_special` = '" . (int)$data['show_as_special'] . "',
                `show_label` = '" . (int)$data['show_label'] . "',
                `discount_method` = '" . (int)$data['discount_method'] . "',
                `discount_amount` = '" . (float)$data['discount_amount'] . "',
                `label` = '" . $this->db->escape(serialize($data['label'])) . "',
                `store` = '" . (empty($data['store_ids']) ? '' : $this->db->escape(serialize($data['store_ids']))) . "',
                `customer_group` = '" . (empty($data['customer_group_ids']) ? '' : $this->db->escape(serialize($data['customer_group_ids']))) . "'"
        );

        $hint_id = (int)$this->db->getLastId();

        $this->cache->delete('sp_hint.' . $hint_id);
        $this->cache->delete('sp_hints');

        return $hint_id;
    }

    public function editHint($hint_id, $data) {

        $this->db->query("
            UPDATE
                " . DB_PREFIX . "sp_hint
            SET
                `name` = '" . $this->db->escape($data['name']) . "',
                `status` = '" . (int)$data['status'] . "',
                `date_start` = '" . $this->db->escape($data['date_start']) . "',
                `date_end` = '" . $this->db->escape($data['date_end']) . "',
                `sort_order` = '" . (int)$data['sort_order'] . "',
                `rule` = '" . $this->db->escape(serialize($data['rule'])) . "',
                `logged` = '" . (int)$data['logged'] . "',
                `show_as_special` = '" . (int)$data['show_as_special'] . "',
                `show_label` = '" . (int)$data['show_label'] . "',
                `discount_method` = '" . (int)$data['discount_method'] . "',
                `discount_amount` = '" . (float)$data['discount_amount'] . "',
                `label` = '" . $this->db->escape(serialize($data['label'])) . "',
                `store` = '" . (empty($data['store_ids']) ? '' : $this->db->escape(serialize($data['store_ids']))) . "',
                `customer_group` = '" . (empty($data['customer_group_ids']) ? '' : $this->db->escape(serialize($data['customer_group_ids']))) . "'
            WHERE hint_id = '" . (int)$hint_id . "'"
        );

        $this->cache->delete('sp_hint.' . (int)$hint_id);
        $this->cache->delete('sp_hints');
    }

    public function deleteHint($hint_id) {
        $this->db->query("DELETE FROM " . DB_PREFIX . "sp_hint WHERE hint_id = '" . (int)$hint_id . "'");

        $this->cache->delete('sp_hint.' . (int)$hint_id);
        $this->cache->delete('sp_hints');
    }

    public function statusHint($hint_id) {
        $this->db->query("
            UPDATE
                " . DB_PREFIX . "sp_hint
            SET
                `status` = 1 - `status`
             WHERE hint_id = '" . (int)$hint_id . "'"
        );

        $this->cache->delete('sp_hint.' . (int)$hint_id);
        $this->cache->delete('sp_hints');
    }

    public function copyHint($hint_id) {
        $this->db->query("
            INSERT INTO
                " . DB_PREFIX . "sp_hint (
                    `name`,
                    `date_start`,
                    `date_end`,
                    `sort_order`,
                    `rule`,
                    `logged`,
                    `discount_method`,
                    `discount_amount`,
                    `label`,
                    `show_as_special`,
                    `show_label`,
                    `store`,
                    `customer_group`
                ) SELECT
                    `name`,
                    `date_start`,
                    `date_end`,
                    `sort_order`,
                    `rule`,
                    `logged`,
                    `discount_method`,
                    `discount_amount`,
                    `label`,
                    `show_as_special`,
                    `show_label`,
                    `store`,
                    `customer_group`
                FROM " . DB_PREFIX . "sp_hint WHERE `hint_id` = '" . (int)$hint_id . "'");
    }

    public function getHint($hint_id) {
        $query = $this->db->query("SELECT *, DATE(date_start) date_start, DATE(date_end) date_end FROM " . DB_PREFIX . "sp_hint WHERE hint_id = '" . (int)$hint_id . "'");

        if ($query->row) {
            $query->row['store_ids'] = $query->row['store'] ? unserialize($query->row['store']) : array();
            $query->row['customer_group_ids'] = $query->row['customer_group'] ? unserialize($query->row['customer_group']) : array();
            $query->row['rule'] = $query->row['rule'] ? unserialize($query->row['rule']) : array();
            $query->row['label'] = unserialize($query->row['label']);
        }

        return $query->row;
    }

    public function arrayRule($rule) {
        if (!empty($rule['conditions'])) {
            $_rule = array();
            foreach ($rule['conditions'] as $positions => $item) {
                $pos = explode('--', $positions);
                $this->putIntoRule($_rule, $pos, $item);
            }
            $rule['conditions'] = $_rule;
        } else {
            $rule['conditions'] = array();
        }

        if (!empty($rule['actions'])) {
            $_rule = array();
            foreach ($rule['actions'] as $positions => $item) {
                $pos = explode('--', $positions);
                $this->putIntoRule($_rule, $pos, $item);
            }
            $rule['actions'] = $_rule;
        } else {
            $rule['actions'] = array();
        }

        return $rule;
    }

    private function putIntoRule(&$rule, $pos, $item) {
        $index = array_shift($pos) - 1;
        if (!count($pos)) {
            $rule[$index] = $item;
        } else {
            if (!isset($rule[$index]['rules'])) {
                $rule[$index]['rules'] = array();
            }
            $this->putIntoRule($rule[$index]['rules'], $pos, $item);
        }
    }
}