<?php
/**
 *  * You are allowed to use this API in your web application.
 *
 * Copyright (C) 2018 by customweb GmbH
 *
 * This program is licenced under the customweb software licence. With the
 * purchase or the installation of the software in your application you
 * accept the licence agreement. The allowed usage is outlined in the
 * customweb software licence which can be found under
 * http://www.sellxed.com/en/software-license-agreement
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at http://www.sellxed.com/shop.
 *
 * See the customweb software licence agreement for more details.
 *
 */

require_once 'Customweb/Payment/Authorization/IInvoiceItem.php';
require_once 'Customweb/Stripe/Method/Default.php';



/**
 *
 * @author Victor Ndambakuwa
 * @Method(paymentMethods={'klarnainvoice'})
 */
class Customweb_Stripe_Method_KlarnaInvoice extends Customweb_Stripe_Method_Default {

	const TYPE_SKU      = 'sku';
	const TYPE_TAX      = 'tax';
	const TYPE_SHIPPING = 'shipping';
	const TYPE_DISCOUNT = 'discount';

	/**
	 * Returns the data to used to create the source, as an array.
	 *
	 * @param \Customweb_Stripe_Authorization_Transaction $transaction
	 *
	 * @return array
	 */
	protected function getSourceCreationData(Customweb_Stripe_Authorization_Transaction $transaction)
	{
		$data                 = parent::getSourceCreationData($transaction);
		$billing_address      = $transaction->getTransactionContext()->getOrderContext()->getBillingAddress();
		$data['klarna']       = [
			'product'          => 'payment',
			'purchase_country' => $billing_address->getCountryIsoCode(),
			'first_name'       => $billing_address->getFirstName(),
			'last_name'        => $billing_address->getLastName(),
		];
		$data['source_order'] = $this->getSourceOrder($transaction);
		$data['flow']         = 'redirect';
		return $data;
	}

	/**
	 * Returns the data to used to create the source order, as an array.
	 *
	 * @param \Customweb_Stripe_Authorization_Transaction $transaction
	 *
	 * @return array
	 */
	protected function getSourceOrder(Customweb_Stripe_Authorization_Transaction $transaction)
	{
		$source_order['items'] = [];
		foreach ($transaction->getTransactionContext()->getOrderContext()->getInvoiceItems() as $invoice_item) {
			$source_order['items'][] = [
				'type'        => $this->getItemType($invoice_item->getType()),
				'description' => $invoice_item->getName(),
				'quantity'    => $invoice_item->getQuantity(),
				'amount'      => $invoice_item->getAmountIncludingTax() * 100,
				'currency'    => $transaction->getCurrencyCode(),
			];
		}
		return $source_order;
	}

	/**
	 * Get item type
	 *
	 * @param string $itemType
	 *
	 * @return string
	 */
	protected function getItemType($itemType)
	{
		$type = self::TYPE_SKU;
		switch ($itemType) {
			case  Customweb_Payment_Authorization_IInvoiceItem::TYPE_DISCOUNT:
				$type = self::TYPE_DISCOUNT;
				break;
			case  Customweb_Payment_Authorization_IInvoiceItem::TYPE_SHIPPING:
				$type = self::TYPE_SHIPPING;
				break;
			case  Customweb_Payment_Authorization_IInvoiceItem::TYPE_FEE:
			case  Customweb_Payment_Authorization_IInvoiceItem::TYPE_PRODUCT:
				$type = self::TYPE_SKU;
				break;
			default:
				$type = self::TYPE_SKU;
		}
		return $type;
	}
}