<?php
/**
*  * You are allowed to use this API in your web application.
 *
 * Copyright (C) 2018 by customweb GmbH
 *
 * This program is licenced under the customweb software licence. With the
 * purchase or the installation of the software in your application you
 * accept the licence agreement. The allowed usage is outlined in the
 * customweb software licence which can be found under
 * http://www.sellxed.com/en/software-license-agreement
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at http://www.sellxed.com/shop.
 *
 * See the customweb software licence agreement for more details.
 *
*/

require_once 'Customweb/Payment/Endpoint/Controller/DelayedNotification.php';
require_once 'Customweb/I18n/Translation.php';
require_once 'Customweb/Stripe/Method/Default.php';
require_once 'Customweb/Stripe/Endpoint/Process.php';



/**
*
* @author Victor Ndambakuwa
* @Method(paymentMethods={'wechatpay'})
*/
class Customweb_Stripe_Method_WeChatPay extends Customweb_Stripe_Method_Default {

	/**
	 * Get We Chat JS
	 *
	 * @param string $successUrl
	 * @return string
	 * @throws Customweb_DependencyInjection_Exception_BeanNotFoundException
	 */
	private function getWeChatFunction($successUrl = '#'){

		$modalCSSUrl = $this->getContainer()->getBean('Customweb_Asset_IResolver')->resolveAssetUrl('cw-stripe-modal.css');
		$iconURL = $this->getContainer()->getBean('Customweb_Asset_IResolver')->resolveAssetUrl('wechatpay.png');
		$scanMeText = Customweb_I18n_Translation::__("Scan me!");

		$JAVASCRIPT = <<<JAVASCRIPT
		
let cssId = 'cw-stripe-modal-id';
if (!document.getElementById(cssId)) {
	let head = document.getElementsByTagName('head')[0];
	let link = document.createElement('link');
	link.id = cssId;
	link.rel = 'stylesheet';
	link.type = 'text/css';
	link.href = '{$modalCSSUrl}';
	link.media = 'all';
	head.appendChild(link);
}

let cwModal = document.getElementById('cw-modal');
if (!cwModal) {
	cwModal = document.createElement('div');
	cwModal.id = 'cw-modal';
	cwModal.className = 'cw-stripe-modal';
	cwModal.innerHTML = '<div class=\'cw-stripe-modal-content\'><div class=\'cw-stripe-modal-header\'><img src="{$iconURL}"/><a id=\'cw-stripe-modal-close\' href="#">&times;</a></div><div id=\'cw-qrcode\'></div><div class=\'cw-stripe-modal-footer\'>{$scanMeText}</div></div>';
	document.body.appendChild(cwModal);
}

let cwModalClose = document.getElementById('cw-stripe-modal-close');
cwModalClose.href = '{$successUrl}';
cwModalClose.onclick = function () {
	cwModal.style.display = 'none';
};
    
new QRCode(document.getElementById('cw-qrcode'), result.source.wechat.qr_code_url);
cwModal.style.display = 'block';

JAVASCRIPT;
		return $JAVASCRIPT;
	}

	/**
	 * Returns the promise called after the creation of the source.
	 * Must return a javascript implementation for function(result).
	 *
	 * @param Customweb_Stripe_Authorization_Transaction $transaction
	 *
	 * @return string
	 * @throws Exception
	 */
	protected function getJavascriptPromiseFunction(Customweb_Stripe_Authorization_Transaction $transaction)
	{
		$failedUrl = $this->getContainer()->getSecuredEndpoint($transaction, 'ajax-failure');
		$context   = $transaction->getTransactionContext();
		/* @var $context Customweb_Payment_Authorization_Ajax_ITransactionContext */
		$failedFunction        = $context->getJavaScriptFailedCallbackFunction();
		$waitUrl               = $this->getContainer()->getSecuredEndpoint($transaction, 'index', 'waiting', '', Customweb_Payment_Endpoint_Controller_DelayedNotification::HASH_PARAMETER);
		$weChatFunction        = $this->getWeChatFunction();
		$successFunction       = $context->getJavaScriptSuccessCallbackFunction();
		$successUrl            = $transaction->getSuccessUrl();
		$errorMessageParameter = Customweb_Stripe_Endpoint_Process::ERROR_MESSAGE_PARAMETER;

		$JAVASCRIPT = <<<JAVASCRIPT

(result) => {
	if(result.error) {
		let suffix = '';
		if(result.error.message) {
			suffix = '&{$errorMessageParameter}=' + encodeURIComponent(result.error.message);
		}
		({$failedFunction})('$failedUrl' + suffix);
	} else {
		{$weChatFunction}
		
		{$this->getPollingJs($transaction, "((" . $successFunction . ")('$successUrl'))")};
	}
}

JAVASCRIPT;
		return $JAVASCRIPT;
	}
}